"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DCircle = void 0;
const DPoint_1 = require("./DPoint");
const DPolygon_1 = require("./DPolygon");
const DNumbers_1 = require("./DNumbers");
class DCircle {
    constructor(center, r) {
        this.center = DPoint_1.DPoint.Zero();
        this.r = 0;
        this.r = r;
        this.center = center;
    }
    toString() {
        return `(${this.center.toString()}, ${this.r})`;
    }
    getValue() {
        return { center: this.center, r: this.r };
    }
    clone() {
        return new DCircle(this.center, this.r);
    }
    /**
     * Find intersection points with other circle.
     * @param c
     */
    findPoints(c) {
        if (this.equal(c)) {
            return Infinity;
        }
        const { center: { x: x0, y: y0 }, r: r0 } = this;
        const { center: { x: x1, y: y1 }, r: r1 } = c;
        const r02 = r0 * r0;
        const d = this.center.distance(c.center);
        const a = (r02 - r1 * r1 + d * d) / (2 * d);
        const h = Math.sqrt(r02 - a * a);
        const ad = a / d;
        const dy = y1 - y0;
        const dx = x1 - x0;
        const hd = h / d;
        const x2 = x0 + ad * (x1 - x0);
        const y2 = y0 + ad * (y1 - y0);
        const x31 = x2 + hd * dy;
        const y31 = y2 - hd * dx;
        const x32 = x2 - hd * dy;
        const y32 = y2 + hd * dx;
        const res = [];
        if (!isNaN(x31) && !isNaN(y31)) {
            res.push(new DPoint_1.DPoint(x31, y31));
        }
        if (!isNaN(x32) && !isNaN(y32) && !(x31 === x32 && y31 === y32)) {
            res.push(new DPoint_1.DPoint(x32, y32));
        }
        return res;
    }
    equal({ center, r }) {
        return this.center.equal(center) && this.r === r;
    }
    findPolygonInside(pointCount = 64) {
        const preAngle = 2 * Math.PI / pointCount;
        const points = [];
        for (let i = 0; i < pointCount; i++) {
            const angle = preAngle * i;
            const x = this.r * Math.cos(angle) + this.center.x;
            const y = this.r * Math.sin(angle) + this.center.y;
            points.push(new DPoint_1.DPoint(x, y));
        }
        return new DPolygon_1.DPolygon([...points, points[0]]);
    }
    findPolygonInsideOnSphere(pointCount = 64) {
        const res = new DPolygon_1.DPolygon();
        for (let i = 0; i < pointCount; i++) {
            res.push(this.sphereOffset(2 * Math.PI * i / pointCount));
        }
        return res;
    }
    sphereOffset(bearing, earthRadius = DPoint_1.EARTH_RADIUS_IN_METERS) {
        const lat1 = DNumbers_1.DNumbers.deg2Rad(this.center.y);
        const lon1 = DNumbers_1.DNumbers.deg2Rad(this.center.x);
        const dByR = this.r / earthRadius;
        const lat = Math.asin(Math.sin(lat1) * Math.cos(dByR) +
            Math.cos(lat1) * Math.sin(dByR) * Math.cos(bearing));
        const lon = lon1 +
            Math.atan2(Math.sin(bearing) * Math.sin(dByR) * Math.cos(lat1), Math.cos(dByR) - Math.sin(lat1) * Math.sin(lat));
        return new DPoint_1.DPoint(DNumbers_1.DNumbers.rad2Deg(lon), DNumbers_1.DNumbers.rad2Deg(lat));
    }
}
exports.DCircle = DCircle;
