"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DLine = void 0;
const DPoint_1 = require("./DPoint");
class DLine {
    constructor(a = 0, b = 0, c = 0, p1 = DPoint_1.DPoint.Zero(), p2 = DPoint_1.DPoint.Zero()) {
        this.a = 0;
        this.b = 0;
        this.c = 0;
        this.p1 = DPoint_1.DPoint.Zero();
        this.p2 = DPoint_1.DPoint.Zero();
        this.a = a;
        this.b = b;
        this.c = c;
        this.p1 = p1;
        this.p2 = p2;
    }
    clone() {
        return new DLine(this.a, this.b, this.c, this.p1.clone(), this.p2.clone());
    }
    findPerpendicular(p) {
        return new DLine(-this.b, this.a, this.b * p.x - this.a * p.y);
    }
    perpendicularDistance(p) {
        const perpendicularLine = this.findPerpendicular(p);
        const targetPoint = perpendicularLine.findPoint(this);
        return targetPoint.distance(p);
    }
    intersection(l, d = 0) {
        const p = this.findPoint(l);
        if (p) {
            return this.inRange(p, d) && l.inRange(p, d) ? p : null;
        }
        return null;
    }
    intersectionWithCircle(circle) {
        const { center, r } = circle;
        const per = this.findPerpendicular(center);
        const t = this.intersection(per, Infinity);
        let distance = t.distance(center);
        if (this.p1.equal(center)) {
            distance = 0;
        }
        if (this.p2.equal(center)) {
            distance = 0;
        }
        if (distance < r) {
            const { a, b, c } = this;
            if (this.isParallel) {
                const ct = center.distance(t);
                const move = Math.sqrt(r * r - ct * ct);
                // Mean "|" x = const
                if (this.isParallelY) {
                    t.x = this.p1.x;
                    const r1 = t.clone().move(0, -move);
                    const r2 = t.clone().move(0, move);
                    return [r1, r2];
                }
                // Mean "-" y = const
                if (this.isParallelX) {
                    t.y = this.p1.y;
                    const r1 = t.clone().move(move, 0);
                    const r2 = t.clone().move(-move, 0);
                    return [r1, r2];
                }
            }
            if (this.p1.like(center)) {
                const p = this.p1.clone();
                return [this.movePoint(p, r), this.movePoint(p, -r)];
            }
            if (this.p2.like(center)) {
                const p = this.p2.clone();
                return [this.movePoint(p, r), this.movePoint(p, -r)];
            }
            const s = a * a + b * b;
            const d = r * r - c * c / s;
            const mult = Math.sqrt(d / s);
            const r1 = t.clone().move(b * mult, -a * mult);
            const r2 = t.clone().move(-b * mult, a * mult);
            return [r1, r2];
        }
        if (distance === r) {
            return t;
        }
        return null;
    }
    inRange(p, d = 0) {
        const { minX, minY, maxX, maxY } = this;
        const { x, y } = p;
        const isInX = (x >= minX && x <= maxX) || Math.abs(minX - x) < d || Math.abs(maxX - x) < d;
        const isInY = (y >= minY && y <= maxY) || Math.abs(minY - y) < d || Math.abs(maxY - y) < d;
        return isInX && isInY;
    }
    get minX() {
        return Math.min(this.p1.x, this.p2.x);
    }
    get minY() {
        return Math.min(this.p1.y, this.p2.y);
    }
    get maxX() {
        return Math.max(this.p1.x, this.p2.x);
    }
    get maxY() {
        return Math.max(this.p1.y, this.p2.y);
    }
    toString() {
        return `(${this.a}, ${this.b}, ${this.c})`;
    }
    getValue() {
        return [this.a, this.b, this.c];
    }
    x(p) {
        if (this.isParallelY) {
            return new DPoint_1.DPoint(-this.c / this.a, p.y);
        }
        if (this.isParallelX) {
            return new DPoint_1.DPoint(p.x, -this.c / this.b);
        }
        return new DPoint_1.DPoint(-this.b / this.a * p.y - this.c / this.a, p.y);
    }
    y(p) {
        if (this.isParallelY) {
            return new DPoint_1.DPoint(-this.c / this.a, p.y);
        }
        if (this.isParallelX) {
            return new DPoint_1.DPoint(p.x, -this.c / this.b);
        }
        return new DPoint_1.DPoint(p.x, -this.a / this.b * p.x - this.c / this.b);
    }
    findPoint(l) {
        if (this.isParallelY && l.isParallelY) {
            return null;
        }
        if (this.isParallelX && l.isParallelX) {
            return null;
        }
        if (this.isParallelX && l.isParallelY) {
            return new DPoint_1.DPoint(-l.c / l.a, -this.c / this.b);
        }
        if (this.isParallelY && l.isParallelX) {
            return new DPoint_1.DPoint(-this.c / this.a, -l.c / l.b);
        }
        if (this.isParallelY) {
            const x = -this.c / this.a;
            return l.y(new DPoint_1.DPoint(x));
        }
        if (this.isParallelX) {
            const y = -this.c / this.b;
            return l.x(new DPoint_1.DPoint(0, y));
        }
        if (l.isParallelY) {
            const x = -l.c / l.a;
            return this.y(new DPoint_1.DPoint(x));
        }
        if (l.isParallelX) {
            const y = -l.c / l.b;
            return this.x(new DPoint_1.DPoint(0, y));
        }
        const res = this.y(new DPoint_1.DPoint((l.c / l.b - this.c / this.b) / (this.a / this.b - l.a / l.b)));
        if (!isFinite(res.x) && !isFinite(res.y)) {
            return null;
        }
        return res;
    }
    get isParallel() {
        return this.isParallelX || this.isParallelY;
    }
    get isParallelY() {
        return Math.abs(this.b) < 0.001;
    }
    get isParallelX() {
        return Math.abs(this.a) < 0.001;
    }
    get points() {
        return [this.p1, this.p2];
    }
    getFi() {
        const l = new DLine(0, 1, 0);
        const result = this.findFi(l);
        if (this.p1.x === this.p2.x) {
            if (this.p1.y === this.p2.y) {
                return 0;
            }
            else if (this.p1.y < this.p2.y) {
                return Math.PI + Math.PI / 2;
            }
            return Math.PI / 2;
        }
        else if (this.p1.x < this.p2.x) {
            if (this.p1.y === this.p2.y) {
                return 0;
            }
            else if (this.p1.y < this.p2.y) {
                return Math.PI - result + Math.PI;
            }
            return result;
        }
        if (this.p1.y === this.p2.y) {
            return Math.PI;
        }
        else if (this.p1.y < this.p2.y) {
            return Math.PI - result + Math.PI;
        }
        return result;
    }
    toWKT() {
        const { p1: { x: x1, y: y1 }, p2: { x: x2, y: y2 } } = this;
        return `LINESTRING (${x1} ${y1}, ${x2} ${y2})`;
    }
    movePoint(p, d) {
        const fi = this.findFi(new DLine(1, 0, 0));
        const td = this.p1.distance(this.p2) / 2;
        const dcosT = td * Math.cos(fi);
        const dsinT = td * Math.sin(fi);
        const p1T = new DPoint_1.DPoint(p.x - dsinT, p.y - dcosT);
        const p2T = new DPoint_1.DPoint(p.x + dsinT, p.y + dcosT);
        const dcos = d * Math.cos(fi);
        const dsin = d * Math.sin(fi);
        const p2 = new DPoint_1.DPoint(p.x + dsin, p.y + dcos);
        const p3 = new DPoint_1.DPoint(p.x - dsin, p.y + dcos);
        if (this.inRange(p1T) || this.inRange(p2T)) {
            return p2;
        }
        return p3;
    }
    findFi(l, delta = 1.0001) {
        let val = (this.a * l.a + this.b * l.b) / (Math.sqrt(this.a * this.a + this.b * this.b) * Math.sqrt(l.a * l.a + l.b * l.b));
        if (val > 1 && val < delta) {
            val = 1;
        }
        else if (val < -1 && val > -delta) {
            val = -1;
        }
        return Math.acos(val);
    }
    vectorProduct(l) {
        const i = this.b * l.c - this.c * l.b;
        const j = this.c * l.a - this.a * l.c;
        const k = this.a * l.b - this.b * l.a;
        return new DLine(i, j, k);
    }
}
exports.DLine = DLine;
