import { DCoord, DPoint, LatLng } from './DPoint';
import { DLine } from './DLine';
interface ParseProps {
    dataProjection: string;
    featureProjection: string;
}
export declare const MIN_POINTS_IN_VALID_POLYGON = 3;
export declare class DPolygon {
    properties: {
        [key: string]: any;
    };
    holes: DPolygon[];
    private pPoints;
    private searchStore;
    constructor(points?: DPoint[]);
    static minAreaRectangleSize(poly: DPolygon): DPoint;
    static toDash(poly: DPolygon): DPolygon[];
    static minAreaRectangleDirection(poly: DPolygon): number;
    static parseFromWKT(wkt: string, optProps?: ParseProps): DPolygon;
    static createSquareBySize(size: DPoint): DPolygon;
    set points(p: DPoint[]);
    get points(): DPoint[];
    get maxX(): number;
    get minX(): number;
    get maxY(): number;
    get minY(): number;
    get center(): DPoint;
    get h(): number;
    get w(): number;
    get dY(): number;
    get dX(): number;
    get extend(): DPolygon;
    get size(): DPoint;
    get leftTop(): DPoint;
    get rightBottom(): DPoint;
    get length(): number;
    get fullLength(): number;
    get perimeter(): number;
    get area(): number;
    get deintersection(): DPolygon;
    get valid(): boolean;
    get first(): DPoint;
    get second(): DPoint;
    get last(): DPoint;
    get minAreaRectangle(): DPolygon;
    get convex(): DPolygon;
    get isClockwise(): boolean;
    get clockWise(): DPolygon;
    get noHoles(): DPolygon;
    intersection(l: DLine): DPoint[];
    setCenter(newCenter: DPoint): DPolygon;
    toWKT(): string;
    rotate(a: number): DPolygon;
    filter(f: (p: DPoint) => boolean): DPolygon;
    move(x?: number | DPoint, y?: number): DPolygon;
    scale(x?: number | DPoint, y?: number): DPolygon;
    divide(x?: number | DPoint, y?: number): DPolygon;
    round(): DPolygon;
    floor(): DPolygon;
    ceil(): DPolygon;
    toFixed(n?: number): DPolygon;
    map(f: (r: DPoint, index?: number) => DPoint): DPolygon;
    p(index: number, divide?: boolean): DPoint;
    pop(): DPoint;
    push(...args: DPoint[]): number;
    shift(): DPoint;
    unshift(...args: DPoint[]): number;
    reverse(): DPolygon;
    getValue(): string;
    transform(from?: string, to?: string): DPolygon;
    toString(): string;
    close(): DPolygon;
    open(): DPolygon;
    height(z: number): DPolygon;
    add(poly: DPolygon): DPolygon;
    has(p: DPoint): boolean;
    clone(): DPolygon;
    equal(p: DPolygon | null): boolean;
    same(p: DPolygon): boolean;
    findIndex(p: DPoint): number;
    approximation(e?: number): DPolygon;
    insertAfter(index: number, ...points: DPoint[]): void;
    removePart(index: number, count: number): DPoint[];
    hasSimpleIntersection(p: DPolygon): boolean;
    simpleInclude(p: DPoint): boolean;
    drawPolygonOnCanvas(canvas: HTMLCanvasElement, fillColor?: string, strokeColor?: string, shadowColor?: string, lineWidth?: number, steps?: number): void;
    clearPolygonOnCanvas(canvas: HTMLCanvasElement): void;
    contain(p: DPoint, isBorderInside?: boolean, move?: DPoint): boolean;
    onBorder(p: DPoint): boolean;
    nextStart(): DPolygon;
    removeDuplicates(): DPolygon;
    static parse(a: LatLng[] | number[][] | DCoord[]): DPolygon;
    toArrayOfCoords(): DCoord[];
    divideToPieces(piecesCount: number): DPolygon;
    prepareToFastSearch(): void;
    fastHas({ x, y, z }: DPoint): boolean;
    get growingPiecesGenerator(): () => Generator<DPolygon, DPolygon>;
    private simpleIncludeX;
    private simpleIncludeY;
    private douglasPeucker;
    private goByPath;
    private contain2;
}
export {};
