"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TraceMatrix = exports.TraceMatrixValues = void 0;
const DPoint_1 = require("./DPoint");
const DPolygon_1 = require("./DPolygon");
const FastSearch_1 = require("./FastSearch");
const createArray = (v = 0) => new Array(v + 1)
    .join(' ')
    .split('')
    .map((r, i) => i);
var TraceMatrixValues;
(function (TraceMatrixValues) {
    TraceMatrixValues[TraceMatrixValues["f"] = 0] = "f";
    TraceMatrixValues[TraceMatrixValues["t"] = 1] = "t";
})(TraceMatrixValues = exports.TraceMatrixValues || (exports.TraceMatrixValues = {}));
const getByPosition = (m, p, defaultValue = TraceMatrixValues.f) => {
    if (m[p.y] === undefined || m[p.y][p.x] === undefined) {
        return defaultValue;
    }
    return m[p.y][p.x];
};
const setByPosition = (m, p, value) => {
    if (m[p.y] === undefined || m[p.y][p.x] === undefined) {
        return value;
    }
    m[p.y][p.x] = value;
    return m[p.y][p.x];
};
class TraceMatrix {
    constructor(size, f) {
        this.findGroupByIndex = (m, s) => {
            const res = new DPolygon_1.DPolygon();
            if (s && getByPosition(m, s) === TraceMatrixValues.t) {
                res.push(s);
                let startIndex = 0;
                const marked = TraceMatrix.createMatrix(this.size, () => TraceMatrixValues.f);
                setByPosition(marked, s, TraceMatrixValues.t);
                while (startIndex < res.length) {
                    const r = res.p(startIndex);
                    for (let i = -1; i < 2; i++) {
                        for (let j = -1; j < 2; j++) {
                            const t = new DPoint_1.DPoint(r.x + i, r.y + j);
                            if (getByPosition(marked, t, TraceMatrixValues.t) === TraceMatrixValues.f &&
                                getByPosition(m, t, TraceMatrixValues.f) === TraceMatrixValues.t) {
                                res.push(t);
                                setByPosition(marked, t, TraceMatrixValues.t);
                            }
                        }
                    }
                    startIndex++;
                }
            }
            return res;
        };
        this.findAllGroupsInMatrix = (m) => {
            const firstMark = this.findMarked(m);
            if (!firstMark) {
                return [];
            }
            const group = this.findGroupByIndex(m, firstMark);
            const groups = [group];
            let groupSum = group.length;
            let allGroups = [...group.points];
            const fs = new FastSearch_1.FastSearch();
            fs.add(allGroups);
            while (groupSum < this.totalCountInMatrix(m)) {
                let mark = this.findMarked(m);
                while (mark && fs.find(mark)) {
                    mark = this.findMarked(m, mark);
                }
                const nextGroup = this.findGroupByIndex(m, mark);
                groupSum += nextGroup.length;
                allGroups = [...allGroups, ...nextGroup.points];
                fs.add(nextGroup.points);
                groups.push(nextGroup);
            }
            return groups.filter((g) => g.length > 2);
        };
        this.traceGroup = (m, group) => {
            const traceDirections = [
                new DPoint_1.DPoint(-1, -1),
                new DPoint_1.DPoint(-1, 0),
                new DPoint_1.DPoint(-1, 1),
                new DPoint_1.DPoint(0, 1),
                new DPoint_1.DPoint(1, 1),
                new DPoint_1.DPoint(1, 0),
                new DPoint_1.DPoint(1, -1),
                new DPoint_1.DPoint(0, -1)
            ];
            const left = (d) => (d + traceDirections.length + 1) % traceDirections.length;
            const right = (d) => (d + traceDirections.length - 1) % traceDirections.length;
            if (group.length < 2) {
                const t = group.p(0).clone();
                return new DPolygon_1.DPolygon([t, t, t]);
            }
            const points = new DPolygon_1.DPolygon();
            let direction = 0;
            let prevDirection = Infinity;
            let p = group.p(0);
            while (!p.equal(group.p(0)) || points.length < 2) {
                // eslint-disable-next-line no-constant-condition
                while (true) {
                    const nextValue = getByPosition(m, p.clone().move(traceDirections[direction]));
                    const nextNeighbourValue = getByPosition(m, p.clone().move(traceDirections[left(direction)]));
                    if (nextValue === TraceMatrixValues.t && nextNeighbourValue === TraceMatrixValues.f) {
                        break;
                    }
                    direction = right(direction);
                }
                if (prevDirection !== direction) {
                    points.push(p);
                    prevDirection = direction;
                }
                p = p.clone().move(traceDirections[direction]);
                direction = left(left(direction));
            }
            return points.approximation().close();
        };
        this.createHoleMatrix = (group) => {
            const fullTraceDirections = [
                new DPoint_1.DPoint(-1, 0),
                new DPoint_1.DPoint(0, 1),
                new DPoint_1.DPoint(1, 0),
                new DPoint_1.DPoint(0, -1)
            ];
            group.prepareToFastSearch();
            const tmpMatrix = TraceMatrix
                .createMatrix(this.size, (p) => group.fastHas(p) ? TraceMatrixValues.t : TraceMatrixValues.f);
            const startCoords = new DPolygon_1.DPolygon();
            for (let i = 0; i < this.size.w; i++) {
                startCoords.push(new DPoint_1.DPoint(i, -1));
                startCoords.push(new DPoint_1.DPoint(i, this.size.h));
            }
            for (let i = 0; i < this.size.h; i++) {
                startCoords.push(new DPoint_1.DPoint(-1, i));
                startCoords.push(new DPoint_1.DPoint(this.size.w, i));
            }
            while (startCoords.length) {
                const point = startCoords.pop();
                for (const direction of fullTraceDirections) {
                    const tmpPoint = point.clone().move(direction);
                    const value = getByPosition(tmpMatrix, tmpPoint, TraceMatrixValues.t);
                    if (value === TraceMatrixValues.f) {
                        setByPosition(tmpMatrix, tmpPoint, TraceMatrixValues.t);
                        startCoords.push(tmpPoint);
                    }
                }
            }
            const t = this.reverseMatrix(tmpMatrix);
            return this.totalCountInMatrix(t) ? t : null;
        };
        this.size = size;
        this.m = TraceMatrix.createMatrix(this.size, f);
    }
    fullMatrixTrace() {
        const groups = this.findAllGroupsInMatrix(this.m);
        const paths = groups.map((g) => this.traceGroup(this.m, g));
        const holeMatrixs = groups.map(this.createHoleMatrix);
        const holesGroups = holeMatrixs.map((m) => m && this.findAllGroupsInMatrix(m));
        const holesPaths = holesGroups.map((hg, index) => hg && hg.map((g) => this
            .traceGroup(holeMatrixs[index], g)).filter((r) => r.length > DPolygon_1.MIN_POINTS_IN_VALID_POLYGON));
        return groups.map((g, index) => {
            const res = paths[index];
            if (holesGroups[index] && holesGroups[index].length) {
                res.holes = holesPaths[index];
            }
            return res;
        });
    }
    reverseMatrix(m) {
        return TraceMatrix.createMatrix(this.size, (p) => getByPosition(m, p) === TraceMatrixValues.f ? TraceMatrixValues.t : TraceMatrixValues.f);
    }
    findMarked(m, init) {
        const s = this.size;
        let ini = false;
        for (let i = 0; i < s.w; i++) {
            for (let j = 0; j < s.h; j++) {
                if (!ini && init) {
                    i = init.x;
                    j = init.y;
                    ini = true;
                    continue;
                }
                if (getByPosition(m, new DPoint_1.DPoint(i, j)) === TraceMatrixValues.t) {
                    return new DPoint_1.DPoint(i, j);
                }
            }
        }
        return null;
    }
    totalCountInMatrix(m) {
        let res = 0;
        const s = this.size;
        for (let i = 0; i < s.w; i++) {
            for (let j = 0; j < s.h; j++) {
                if (getByPosition(m, new DPoint_1.DPoint(i, j))) {
                    res++;
                }
            }
        }
        return res;
    }
    static createMatrix(size, f = () => TraceMatrixValues.f) {
        return createArray(size.h).map((i) => createArray(size.w).map((j) => f(new DPoint_1.DPoint(j, i))));
    }
}
exports.TraceMatrix = TraceMatrix;
